@@ -18,10 +18,15 @@ use strict;
 use MIME::Base64;
 use Exporter 'import';
 use Safe;
+use Lemonldap::NG::Common::Safelib;    #link protected safe Safe object
 require Data::Dumper;
 require POSIX;
 
-our $VERSION = '0.9';
+#inherits Cache::Cache
+#inherits Apache::Session
+#link Lemonldap::NG::Common::Apache::Session::SOAP protected globalStorage
+
+our $VERSION = '0.92';
 
 our %EXPORT_TAGS;
 
@@ -31,14 +36,14 @@ our @EXPORT;
 
 # Shared variables
 our (
-    $locationRegexp,      $locationCondition,    $defaultCondition,
-    $forgeHeaders,        $apacheRequest,        $locationCount,
-    $cookieName,          $portal,               $datas,
-    $globalStorage,       $globalStorageOptions, $localStorage,
-    $localStorageOptions, $whatToTrace,          $https,
-    $refLocalStorage,     $safe,                 $cookieSecured,
-    $port,                $statusPipe,           $statusOut,
-    $customFunctions,
+    $locationRegexp,       $locationCondition, $defaultCondition,
+    $forgeHeaders,         $apacheRequest,     $locationCount,
+    $cookieName,           $datas,             $globalStorage,
+    $globalStorageOptions, $localStorage,      $localStorageOptions,
+    $whatToTrace,          $https,             $refLocalStorage,
+    $safe,                 $port,              $statusPipe,
+    $statusOut,            $customFunctions,   $transform,
+    $cda,
 );
 
 ##########################################
@@ -53,21 +58,20 @@ BEGIN {
         locationRules => [
             qw(
               $locationCondition $defaultCondition $locationCount
-              $locationRegexp $apacheRequest $datas $safe $portal
-              safe $customFunctions
+              $locationRegexp $apacheRequest $datas safe $customFunctions
               )
         ],
         import  => [qw( import @EXPORT_OK @EXPORT %EXPORT_TAGS )],
         headers => [
             qw(
               $forgeHeaders lmHeaderIn lmSetHeaderIn lmHeaderOut
-              lmSetHeaderOut lmSetErrHeaderOut $cookieName $cookieSecured
-              $https $port
+              lmSetHeaderOut lmSetErrHeaderOut $cookieName $https $port
               )
         ],
-        log    => [qw(lmSetApacheUser)],
         traces => [qw( $whatToTrace $statusPipe $statusOut )],
         apache => [qw( MP OK REDIRECT FORBIDDEN DONE DECLINED SERVER_ERROR )],
+        post   => [qw($transform)],
+        cda    => ['$cda'],
     );
     push( @EXPORT_OK, @{ $EXPORT_TAGS{$_} } ) foreach ( keys %EXPORT_TAGS );
     $EXPORT_TAGS{all} = \@EXPORT_OK;
@@ -114,7 +118,6 @@ BEGIN {
             threads::shared::share($forgeHeaders);
             threads::shared::share($locationCount);
             threads::shared::share($cookieName);
-            threads::shared::share($portal);
             threads::shared::share($globalStorage);
             threads::shared::share($globalStorageOptions);
             threads::shared::share($localStorage);
@@ -178,6 +181,7 @@ sub logout_mp2 : method {
 # @param $level string (debug, info, warning or error)
 sub lmLog {
     my ( $class, $mess, $level ) = @_;
+    die "Level is required" unless ($level);
     if ( MP() == 2 ) {
         Apache2::ServerRec->log->$level($mess);
     }
@@ -189,12 +193,12 @@ sub lmLog {
     }
 }
 
-## @rfn protected void lmSetApacheUser(Apache2::RequestRec r,string s)
+## @rmethod protected void lmSetApacheUser(Apache2::RequestRec r,string s)
 # Inform Apache for the data to use as user for logs
 # @param $r current request
 # @param $s string to use
 sub lmSetApacheUser {
-    my ( $r, $s ) = @_;
+    my ( $class, $r, $s ) = @_;
     return unless ($s);
     if ( MP() == 2 ) {
         $r->user($s);
@@ -210,7 +214,7 @@ sub lmSetApacheUser {
 # @return string
 sub regRemoteIp {
     my ( $class, $str ) = @_;
-    if( MP() == 2 ) {
+    if ( MP() == 2 ) {
         $str =~ s/\$datas->\{ip\}/\$apacheRequest->connection->remote_ip/g;
     }
     else {
@@ -234,7 +238,7 @@ sub lmSetHeaderIn {
     }
 }
 
-## @rfn string lmtHeaderIn(Apache2::RequestRec r, string h)
+## @rfn string lmHeaderIn(Apache2::RequestRec r, string h)
 # Return an HTTP header value from the HTTP request.
 # @param $r Current request
 # @param $h Name of the header
@@ -279,7 +283,7 @@ sub lmSetHeaderOut {
     }
 }
 
-## @rfn string lmtHeaderOut(Apache2::RequestRec r, string h)
+## @rfn string lmHeaderOut(Apache2::RequestRec r, string h)
 # Return an HTTP header value from the HTTP response.
 # @param $r Current request
 # @param $h Name of the header
@@ -312,7 +316,7 @@ sub statusProcess {
         $statusOut->writer();
         my $fdin  = $statusPipe->fileno;
         my $fdout = $statusOut->fileno;
-        open STDIN, "<&$fdin";
+        open STDIN,  "<&$fdin";
         open STDOUT, ">&$fdout";
         my @tmp = ();
         push @tmp, "-I$_" foreach (@INC);
@@ -321,8 +325,7 @@ sub statusProcess {
           '-e',
           '&Lemonldap::NG::Handler::Status::run('
           . $localStorage . ','
-          . Data::Dumper->new( [$localStorageOptions] )->Terse(1)->Dump
-          . ');';
+          . Data::Dumper->new( [$localStorageOptions] )->Terse(1)->Dump . ');';
     }
 }
 
@@ -335,28 +338,34 @@ sub statusProcess {
 # @return Safe object
 sub safe {
     my $class = shift;
-    return $safe if($safe);
-    $safe = new Safe;
+    no strict 'refs';
+    return ${"$class\::safe"} if (${"$class\::safe"});
+    $class->lmLog( "Compiling safe jail for $class", 'debug' );
+    my $safe = new Safe;
     my @t = $customFunctions ? split( /\s+/, $customFunctions ) : ();
-    foreach(@t) {
-        $class->lmLog("Custom function : $_",'debug');
+    foreach (@t) {
+        $class->lmLog( "Custom function : $_", 'debug' );
         my $sub = $_;
-        unless(/::/) {
+        unless (/::/) {
             $sub = "$class\::$_";
         }
         else {
             s/^.*:://;
         }
-        next if($class->can($_));
+        next if ( $class->can($_) );
         eval "sub $_ {
             return $sub(\$apacheRequest->uri
                 . ( \$apacheRequest->args ? '?' . \$apacheRequest->args : '' )
                 , \@_)
             }";
-        $class->lmLog($@,'error')if($@);
+        $class->lmLog( $@, 'error' ) if ($@);
     }
-    $safe->share( '&encode_base64', '$datas', '&lmSetHeaderIn', '$apacheRequest', @t );
-    return $safe;
+    $safe->share_from( 'main', ['%ENV'] );
+    $safe->share_from( 'Lemonldap::NG::Common::Safelib',
+        $Lemonldap::NG::Common::Safelib::functions );
+    $safe->share( '&encode_base64', '$datas', '&lmSetHeaderIn',
+        '$apacheRequest', '&portal', @t );
+    return ${"$class\::safe"} = $safe;
 }
 
 ## @imethod void init(hashRef args)
@@ -404,17 +413,11 @@ sub childInit {
     if ( MP() == 2 ) {
         Apache2::ServerUtil->server->push_handlers( PerlChildInitHandler =>
               sub { return $class->initLocalStorage( $_[1], $_[0] ); } );
-        Apache2::ServerUtil->server->push_handlers(
-            PerlCleanupHandler => sub { return $class->cleanLocalStorage(@_); }
-        );
     }
     elsif ( MP() == 1 ) {
         Apache->push_handlers(
             PerlChildInitHandler => sub { return $class->initLocalStorage(@_); }
         );
-        Apache->push_handlers(
-            PerlCleanupHandler => sub { return $class->cleanLocalStorage(@_); }
-        );
     }
     1;
 }
@@ -429,9 +432,7 @@ sub purgeCache {
 
     # At each Apache (re)start, we've to clear the cache to avoid living
     # with old datas
-    eval '$refLocalStorage = new '
-      . $localStorage
-      . '($localStorageOptions);';
+    eval '$refLocalStorage = new ' . $localStorage . '($localStorageOptions);';
     if ( defined $refLocalStorage ) {
         $refLocalStorage->clear();
     }
@@ -450,11 +451,12 @@ sub purgeCache {
 # @param $args reference to the configuration hash
 sub globalInit {
     my $class = shift;
+    $class->portalInit(@_);
     $class->locationRulesInit(@_);
     $class->defaultValuesInit(@_);
-    $class->portalInit(@_);
     $class->globalStorageInit(@_);
     $class->forgeHeadersInit(@_);
+    $class->postUrlInit(@_);
 }
 
 ## @imethod protected void locationRulesInit(hashRef args)
@@ -473,7 +475,7 @@ sub locationRulesInit {
     $locationCount = 0;
 
     # Pre compilation : both regexp and conditions
-    foreach ( keys %{ $args->{locationRules} } ) {
+    foreach ( sort keys %{ $args->{locationRules} } ) {
         if ( $_ eq 'default' ) {
             $defaultCondition =
               $class->conditionSub( $args->{locationRules}->{$_} );
@@ -502,13 +504,15 @@ sub conditionSub {
       if ( $cond =~ /^accept$/i );
     return sub { 0 }
       if ( $cond =~ /^deny$/i );
-    if ( $cond =~ /^logout(?:_sso)?(?:\s+(.*))$/i ) {
+    if ( $cond =~ /^logout(?:_sso)?(?:\s+(.*))?$/i ) {
         my $url = $1;
-        return sub { $datas->{_logout} = $url; return 0 }
+        return $url
+          ? sub { $datas->{_logout} = $url;     return 0 }
+          : sub { $datas->{_logout} = portal(); return 0 };
     }
     if ( MP() == 2 ) {
         if ( $cond =~ /^logout_app(?:\s+(.*))?$/i ) {
-            my $u = $1;
+            my $u = $1 || 'portal()';
             eval 'use Apache2::Filter' unless ( $INC{"Apache2/Filter.pm"} );
             return sub {
                 $apacheRequest->add_output_filter(
@@ -521,13 +525,13 @@ sub conditionSub {
         }
         elsif ( $cond =~ /^logout_app_sso(?:\s+(.*))?$/i ) {
             eval 'use Apache2::Filter' unless ( $INC{"Apache2/Filter.pm"} );
-            my $u = $1;
+            my $u = $1 || 'portal()';
             return sub {
                 $class->localUnlog;
                 $apacheRequest->add_output_filter(
                     sub {
                         return $class->redirectFilter(
-                            "$portal?url="
+                            $class->portal() . "?url="
                               . $class->encodeUrl($u)
                               . "&logout=1",
                             @_
@@ -540,8 +544,8 @@ sub conditionSub {
     }
     $cond =~ s/\$date/&POSIX::strftime("%Y%m%d%H%M%S",localtime())/e;
     $cond =~ s/\$(\w+)/\$datas->{$1}/g;
-    my $sub;
-    $sub = $class->safe->reval("sub {return ( $cond )}");
+    $cond =~ s/\$datas->{vhost}/\$apacheRequest->hostname/g;
+    my $sub = $class->safe->reval("sub {return ( $cond )}");
     return $sub;
 }
 
@@ -552,14 +556,16 @@ sub defaultValuesInit {
     my ( $class, $args ) = @_;
 
     # Other values
-    $cookieName    = $args->{cookieName}    || 'lemonldap';
-    $cookieSecured = $args->{cookieSecured} || 0;
-    $whatToTrace   = $args->{whatToTrace}   || '$uid';
+    $cookieName  = $args->{cookieName}  || $cookieName  || 'lemonldap';
+    $whatToTrace = $args->{whatToTrace} || $whatToTrace || 'uid';
     $whatToTrace =~ s/\$//g;
     $https = $args->{https} unless defined($https);
     $https = 1 unless defined($https);
-    $port = $args->{port} || 0 unless defined($port);
+    $args->{securedCookie} = 1 unless defined( $args->{securedCookie} );
+    $cookieName .= 'http' if ( $args->{securedCookie} == 2 and $https == 0 );
+    $port            = $args->{port} || 0 unless defined($port);
     $customFunctions = $args->{customFunctions};
+    $cda             = $args->{cda} || 0 unless defined($cda);
     1;
 }
 
@@ -568,7 +574,16 @@ sub defaultValuesInit {
 # @param $args reference to the configuration hash
 sub portalInit {
     my ( $class, $args ) = @_;
-    $portal = $args->{portal} or die("portal parameter required");
+    die("portal parameter required") unless ( $args->{portal} );
+    if ( $args->{portal} =~ /[\$\(&\|"']/ ) {
+        my $portal = $class->conditionSub( $args->{portal} );
+        eval "sub portal {return &\$portal}";
+    }
+    else {
+        eval "sub portal {return '$args->{portal}'}";
+    }
+    die("Unable to read portal parameter ($@)") if ($@);
+    1;
 }
 
 ## @imethod protected void globalStorageInit(hashRef args)
@@ -620,13 +635,84 @@ sub forgeHeadersInit {
 sub initLocalStorage {
     my ( $class, $r ) = @_;
     if ( $localStorage and not $refLocalStorage ) {
-        eval "use $localStorage;\$refLocalStorage = new $localStorage(\$localStorageOptions);";
+        eval
+"use $localStorage;\$refLocalStorage = new $localStorage(\$localStorageOptions);";
         $class->lmLog( "Local cache initialization failed: $@", 'error' )
           unless ( defined $refLocalStorage );
     }
     return DECLINED;
 }
 
+## @imethod protected void postUrlInit()
+# Prepare methods to post form attributes
+sub postUrlInit {
+    my ( $class, $args ) = @_;
+    return unless ( $args->{post} );
+    eval 'use Apache2::Filter;use URI';
+    $transform = {};
+    while ( my ( $url, $d ) = each( %{ $args->{post} } ) ) {
+        $d->{postUrl} ||= $url;
+        $transform->{ $d->{postUrl} } =
+          sub { $class->buildPostForm( $d->{postUrl} ) }
+          if ( $url ne $d->{postUrl} );
+
+        my $expr = $d->{expr};
+        $expr =~ s/\$(\w+)/\$datas->{$1}/g;
+        my %h = split /(?:\s*=>\s*|\s*,\s*)/, $expr;
+        my $tmp;
+        foreach ( keys %h ) {
+            $h{$_} = "'$h{$_}'" if ( $h{$_} =~ /^\w+$/ );
+            $tmp .= "'$_'=>$h{$_},";
+        }
+        my $sub = $class->safe->reval(
+            "sub{
+            my \$f = shift;
+            my \$l;
+            unless(\$f->ctx){
+            \$f->ctx(1);
+            my \$u=URI->new('http:');
+            \$u->query_form({$tmp});
+            my \$s=\$u->query();
+            \$l = \$f->r->headers_in->{'Content-Length'};
+            \$f->r->headers_in->set( 'Content-Length' => length(\$s) );
+            \$f->r->headers_in->set( 'Content-Type' => 'application/x-www-form-urlencoded' );
+            \$f->print(\$s);
+            while ( \$f->read( my \$b, \$l ) ) {}
+            \$f->seen_eos(1);
+            }
+            return OK;
+        }"
+        );
+        $class->lmLog( "Compiling POST request for $url", 'debug' );
+        $transform->{$url} = sub {
+            return $class->buildPostForm($url)
+              if ( $apacheRequest->method ne 'POST' );
+            $apacheRequest->add_input_filter($sub);
+            OK;
+          }
+    }
+}
+
+sub buildPostForm {
+    my $class = shift;
+    my $url   = shift;
+    my $count = shift || 1000;
+    $apacheRequest->handler("perl-script");
+    $apacheRequest->set_handlers(
+        'PerlResponseHandler' => sub {
+            my $r = shift;
+            $r->content_type('text/html; charset=UTF-8');
+            $r->print(
+qq{<html><body onload="document.getElementById('f').submit()"><form id="f" method="post" action="$url"><input type=hidden name="a" value="}
+                  . sprintf( "%0" . $count . "d", 1 )
+                  . qq{"/><input type="submit" value="Ok"/></form></body></html>}
+            );
+            OK;
+        }
+    );
+    OK;
+}
+
 ###################
 # RUNNING METHODS #
 ###################
@@ -636,7 +722,7 @@ sub initLocalStorage {
 sub updateStatus {
     my ( $class, $user, $url, $action ) = @_;
     eval {
-        print $statusPipe "$user => "
+            print $statusPipe "$user => "
           . $apacheRequest->hostname
           . "$url $action\n"
           if ($statusPipe);
@@ -660,7 +746,7 @@ sub grant {
 # Inform the status processus and call logForbidden().
 # @return Apache2::Const::FORBIDDEN
 sub forbidden {
-    my $class = shift;
+    my ( $class, $uri ) = @_;
     if ( $datas->{_logout} ) {
         $class->updateStatus( $datas->{$whatToTrace}, $_[0], 'LOGOUT' );
         my $u = $datas->{_logout};
@@ -668,31 +754,50 @@ sub forbidden {
         return $class->goToPortal( $u, 'logout=1' );
     }
     $class->updateStatus( $datas->{$whatToTrace}, $_[0], 'REJECT' );
-    $class->logForbidden(@_);
+    $apacheRequest->push_handlers(
+        PerlLogHandler => sub { $class->logForbidden( $uri, $datas ); DECLINED }
+    );
     return FORBIDDEN;
 }
 
-## @rmethod protected void logForbidden()
+## @rmethod protected void logForbidden(string uri,hashref datas)
 # Insert a log in Apache errors log system to inform that the user was rejected.
 # This method has to be overloaded to use different logs systems
+# @param $uri uri asked
+# @param $datas hash re to user's datas
 sub logForbidden {
-    my $class = shift;
+    my ( $class, $uri, $datas ) = @_;
     $class->lmLog(
-        'The user "'
+        'User "'
           . $datas->{$whatToTrace}
           . '" was reject when he tried to access to '
-          . shift,
+          . $uri,
         'notice'
     );
 }
 
+## @rmethod protected void logGranted(string uri)
+# Insert a log in Apache errors log system to inform that the user was
+# authorizated. This method has to be overloaded to use different logs systems
+# @param $uri uri asked
+sub logGranted {
+    my ( $class, $uri, $datas ) = @_;
+    $class->lmLog(
+        'User "'
+          . $datas->{$whatToTrace}
+          . '" was granted to access to '
+          . $uri,
+        'debug'
+    );
+}
+
 ## @rmethod protected void hideCookie()
 # Hide Lemonldap::NG cookie to the protected application.
 sub hideCookie {
     my $class = shift;
     $class->lmLog( "$class: removing cookie", 'debug' );
     my $tmp = lmHeaderIn( $apacheRequest, 'Cookie' );
-    $tmp =~ s/$cookieName[^;]*;?//o;
+    $tmp =~ s/$cookieName[^,;]*[,;]?//o;
     lmSetHeaderIn( $apacheRequest, 'Cookie' => $tmp );
 }
 
@@ -713,9 +818,7 @@ sub encodeUrl {
           . $portString
           . $url;
     }
-    $u = encode_base64($u);
-    $u =~ s/[\r\n\s]//sg;
-    return $u;
+    return encode_base64( $u, '' );
 }
 
 ## @rmethod protected int goToPortal(string url, string arg)
@@ -733,7 +836,9 @@ sub goToPortal {
     );
     my $urlc_init = $class->encodeUrl($url);
     lmSetHeaderOut( $apacheRequest,
-        'Location' => "$portal?url=$urlc_init" . ( $arg ? "&$arg" : "" ) );
+            'Location' => $class->portal()
+          . "?url=$urlc_init"
+          . ( $arg ? "&$arg" : "" ) );
     return REDIRECT;
 }
 
@@ -742,7 +847,7 @@ sub goToPortal {
 # @return Value of the cookie if found, 0 else
 sub fetchId {
     my $t = lmHeaderIn( $apacheRequest, 'Cookie' );
-    return ( $t =~ /$cookieName=([^; ]+);?/o ) ? $1 : 0;
+    return ( $t =~ /$cookieName=([^,; ]+)/o ) ? $1 : 0;
 }
 
 # MAIN SUBROUTINE called by Apache (using PerlHeaderParserHandler option)
@@ -762,17 +867,37 @@ sub fetchId {
 sub run ($$) {
     my $class;
     ( $class, $apacheRequest ) = @_;
-
     return DECLINED unless ( $apacheRequest->is_initial_req );
-    my $uri = $apacheRequest->uri
-      . ( $apacheRequest->args ? "?" . $apacheRequest->args : "" );
+    my $args = $apacheRequest->args;
+
+    # Cross domain authentication
+    if ( $cda and $args =~ s/[\?&]?($cookieName=\w+)$//oi ) {
+        my $str = $1;
+        $class->lmLog( 'CDA request', 'debug' );
+        $apacheRequest->args($args);
+        my $host = $apacheRequest->get_server_name();
+        my $portString = $port || $apacheRequest->get_server_port();
+        lmSetErrHeaderOut( $apacheRequest,
+                'Location' => "http"
+              . ( $https ? 's' : '' )
+              . "://$host:$portString"
+              . $apacheRequest->uri
+              . ( $args ? "?" . $args : "" ) );
+        $host =~ s/^[^\.]+\.(.*\..*$)/$1/;
+        lmSetErrHeaderOut( $apacheRequest,
+            'Set-Cookie' => "$str; domain=$host; path=/"
+              . ( $https ? "; secure" : "" ) );
+        return REDIRECT;
+    }
+    my $uri = $apacheRequest->uri . ( $args ? "?$args" : "" );
 
     # AUTHENTICATION
     # I - recover the cookie
     my $id;
     unless ( $id = $class->fetchId ) {
         $class->lmLog( "$class: No cookie found", 'info' );
-        $class->updateStatus( $apacheRequest->connection->remote_ip, $apacheRequest->uri, 'REDIRECT' );
+        $class->updateStatus( $apacheRequest->connection->remote_ip,
+            $apacheRequest->uri, 'REDIRECT' );
         return $class->goToPortal($uri);
     }
 
@@ -792,7 +917,8 @@ sub run ($$) {
                 # The cookie isn't yet available
                 $class->lmLog( "The cookie $id isn't yet available: $@",
                     'info' );
-                $class->updateStatus( $apacheRequest->connection->remote_ip, $apacheRequest->uri, 'EXPIRED' );
+                $class->updateStatus( $apacheRequest->connection->remote_ip,
+                    $apacheRequest->uri, 'EXPIRED' );
                 return $class->goToPortal($uri);
             }
             $datas->{$_} = $h{$_} foreach ( keys %h );
@@ -807,17 +933,11 @@ sub run ($$) {
 
     # ACCOUNTING
     # 1 - Inform Apache
-    lmSetApacheUser( $apacheRequest, $datas->{$whatToTrace} );
+    $class->lmSetApacheUser( $apacheRequest, $datas->{$whatToTrace} );
 
     # AUTHORIZATION
     return $class->forbidden($uri) unless ( $class->grant($uri) );
     $class->updateStatus( $datas->{$whatToTrace}, $apacheRequest->uri, 'OK' );
-    $class->lmLog(
-        "User "
-          . $datas->{$whatToTrace}
-          . " was authorizated to access to $uri",
-        'debug'
-    );
 
     # ACCOUNTING
     # 2 - Inform remote application
@@ -826,6 +946,18 @@ sub run ($$) {
     # SECURITY
     # Hide Lemonldap::NG cookie
     $class->hideCookie;
+
+    # Cleanup and log
+    $apacheRequest->push_handlers(
+        PerlLogHandler => sub { $class->logGranted( $uri, $datas ); DECLINED },
+    );
+    $apacheRequest->push_handlers(
+        PerlCleanupHandler => sub { $class->cleanLocalStorage(@_); DECLINED },
+    );
+
+    if ( defined( $transform->{$uri} ) ) {
+        return &{ $transform->{$uri} };
+    }
     OK;
 }
 
@@ -870,7 +1002,8 @@ sub unlog ($$) {
     my $class;
     ( $class, $apacheRequest ) = @_;
     $class->localUnlog;
-    $class->updateStatus( $apacheRequest->connection->remote_ip, $apacheRequest->uri, 'LOGOUT' );
+    $class->updateStatus( $apacheRequest->connection->remote_ip,
+        $apacheRequest->uri, 'LOGOUT' );
     return $class->goToPortal( '/', 'logout=1' );
 }
 
@@ -889,14 +1022,22 @@ sub redirectFilter {
         # Here, we can use Apache2 functions instead of lmSetHeaderOut because
         # this function is used only with Apache2.
         $f->r->status(REDIRECT);
-        $f->r->status_line("302 Temporary Moved");
+        $f->r->status_line("303 See Other");
         $f->r->headers_out->unset('Location');
         $f->r->err_headers_out->set( 'Location' => $url );
         $f->ctx(1);
     }
     while ( $f->read( my $buffer, 1024 ) ) {
     }
-    $class->updateStatus( ( $datas->{$whatToTrace} ? $datas->{$whatToTrace} : $f->r->connection->remote_ip ), 'filter', 'REDIRECT' );
+    $class->updateStatus(
+        (
+              $datas->{$whatToTrace}
+            ? $datas->{$whatToTrace}
+            : $f->r->connection->remote_ip
+        ),
+        'filter',
+        'REDIRECT'
+    );
     return REDIRECT;
 }
 
